/* Board display */
#include <stdio.h>
#include "blocks.h"

/* Send ANSI background colour sequence, if needed.
   If background colour = 1, then foreground is set to 0 */
void setBpen0(int newpen)
{
    static int oldpen = 0;

    if (oldpen != newpen)
    {
        if (newpen==1)
            printf(ES "[30;41m");
        else if (oldpen==1)
            printf(ES "[31;%dm", 40+newpen);
        else
            printf(ES "[%dm", 40+newpen);

        oldpen = newpen;
    }
}

void setBpen(int newpen)
{
    static int oldpen = 0;

    if (oldpen != newpen)
    {
        printf(ES "[%dm", 40+newpen);
        oldpen = newpen;
    }
}

/* Send ANSI foreground colour sequence, if needed.
   Used when linetype is not NOLINE */
void setFpen(int newpen)
{
    static int oldpen = 0;

    if (oldpen != newpen)
    {
        printf(ES "[%dm", 30+newpen);
        oldpen = newpen;
    }
}

void setpen(int newpen)
{
    if (colourflag)
    {
        if (lineflag == NOLINE)
            setBpen(newpen);
        else
            setFpen(newpen);
    }
}

void resetpen(void)
{
    if (lineflag == NOLINE)
        setBpen(8);
    else
        setFpen(0);
}

/* Display a cell from the shape */
void putcell(int c)
{
    if (c >= '0')
    {
        setpen(1 + (c - '0') % 7);

        if (charset)
            c = charset[(c-'0') % charsetlen];
        else
            c = ' ';
    }
    else                /* Must be a dot */
        resetpen();

    putchar(c);
}

/* Pointer to board display functions. */
void (*putboard)(void);

/*  Simple board display. */
void putboardNL(void)
{
    int i,j;

    for(j=0; j<bheight; j++)
    {
        for(i=0; i<bwidth; i++)
        {
            putcell(*pbrd(i,j));
            putchar(' ');
        }
        setBpen(0);
        puts(ES "[0m");
    }
    resetpen();
    puts(ES "[0m");
}

/*

Use IBM line drawing characters to draw boxes around the shapes.

If a pair of horizontally or vertically adjacent cells are from different
shapes, they require a border to separate them.  The vertex type is a
number formed by combining the axis numbers for each required border.
(This axis number ordering has been chosen arbitrarily.)

           2
           |
      i,j  | i+1,j
   4-------*---------1
     i,j+1 | i+1,j+1
           |
           8

For example,

    2                     2            2
    |                     |            |
    *--1        *--1      *         4--*
                |         |
                8         8

   = 3         = 9       = 10         = 6


Note that 1, 2, 4 and 8 are not logically possible.

*/

/* Calculate vertex type by comparing adjacent cells */
int vertex(int i, int j)
{
    return (*pbrd(i+1,j) != *pbrd(i+1,j+1))
        | (*pbrd(i,j) != *pbrd(i+1,j)) << 1
        | (*pbrd(i,j) != *pbrd(i,j+1)) << 2
        | (*pbrd(i,j+1) != *pbrd(i+1,j+1)) << 3;
}

/* Some special vertex types. These are also used between cells */
#define SPACE 0
#define HORIZ 5
#define VERT 10

/* Vertex character sets */
char ISOchars[] = " **+*-++*+|+++++";

/* Single lines */
char IBMchars1[] =
{
    ' ', '*', '*', 192, '*', 196, 217, 193,
    '*', 218, 179, 195, 191, 194, 180, 197
};

/* Double lines */
char IBMchars2[] =
{
    ' ', '*', '*', 200, '*', 205, 188, 202,
    '*', 201, 186, 204, 187, 203, 185, 206
};

char *linechars[] =
{
    NULL, ISOchars, IBMchars1, IBMchars2
};

#define putv(n) do {if (n!=SPACE) resetpen();\
    putchar(linechars[lineflag][n]);} while (0)
#define putvertex(i,j) putv(vertex(i,j))

void putborder(int j)
{
    int i, k, n;

    putvertex(-1,j);
    for(i=0; i<bwidth; i++)
    {
        n = *pbrd(i,j) == *pbrd(i,j+1) ? SPACE : HORIZ;
        for (k=0; k<3; k++)
            putv(n);

        putvertex(i,j);
    }

    putchar('\n');
}

void putcells(int j)
{
    int i, n, c;

    putv(VERT);
    for(i=0; i<bwidth; i++)
    {
        putchar(' ');
        c = *pbrd(i,j);

        putcell(c);
        putchar(' ');

        n = c == *pbrd(i+1,j) ? SPACE : VERT;
        putv(n);
    }

    putchar('\n');
}

/* Fancy board display, using line drawing chars */
void putboardL(void)
{
    int j;

    putborder(-1);
    for(j=0; j<bheight; j++)
    {
        putcells(j);
        putborder(j);
    }

    putchar('\n');
}

/* Very simple board display, with no spacing or char conversion */
void putboardQ(void)
{
    int i,j;

    for(j=0; j<bheight; j++)
    {
        for(i=0; i<bwidth; i++)
            putchar(*pbrd(i,j));
        putchar('\n');
    }
    putchar('\n');
}

/* Board output as an array for POV */
void putboardPOV(void)
{
    int i,j;
    
    printf("#declare Polyomino%d = array[%d][%d]\n{\n", solvecount, bheight+2, bwidth+2);
    printf("  {0, ");
    for(i=0; i<bwidth; i++)
        printf("%2d, ", 0);
    printf(" 0},\n");
    
    for(j=0; j<bheight; j++)
    {
        printf("  {0, ");
        for(i=0; i<bwidth; i++)
            printf("%2d, ", *pbrd(i,j) + 1 - '0');
        printf(" 0},\n");
    }
    printf("  {0, ");
    for(i=0; i<bwidth; i++)
        printf("%2d, ", 0);
    printf(" 0},\n");
    printf("}\n\n");
}

